/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/
#   define FILECODE "F(300)"

#include <linux/string.h>       // for memset

#include "vmware.h"
#include "machine.h"
#include "modulecall.h"


/*
 *----------------------------------------------------------------------
 *
 * LinuxDriver_CheckPadding --
 *
 *      Check for expected padding.
 *      This check currently fails on the egcs compiler.
 *      Also check size of crosspage and offset of linearaddress.
 *
 * Results:
 *      TRUE if the check succeeds -- module will be loaded
 *
 * Side effects:
 *      Output to kernel log on error.
 *
 *----------------------------------------------------------------------
 */

Bool
LinuxDriver_CheckPadding(void)
{
   DTRWords dtr;
   uint16 *x;

   memset(&dtr, 0, sizeof dtr);
   dtr.dtr.limit = 0x1111;
   dtr.dtr.offset = 0x22223333;

   x = (uint16*)&dtr;

   if (x[0] == 0x1111 && x[1] == 0x3333 && x[2] == 0x2222) {
   } else {
      Warning("DTR padding\n");
      goto error;
   }

   if (   sizeof (VMCrossPageTOT) != MODULECALL_CROSSPAGE_SIZE_TOT
       || offsetof(VMCrossPageTOT, crosspageLinearAddr)
             != MODULECALL_CROSSPAGE_LAOFFSET_TOT) {
      Warning("cross page 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof (VMCrossPageTOT), MODULECALL_CROSSPAGE_SIZE_TOT,
              offsetof(VMCrossPageTOT, crosspageLinearAddr),
              MODULECALL_CROSSPAGE_LAOFFSET_TOT);
      goto error;
   }

   if (   sizeof (VMCrossPageV45) != MODULECALL_CROSSPAGE_SIZE_V45
       || offsetof(VMCrossPageV45, crosspageLinearAddr)
             != MODULECALL_CROSSPAGE_LAOFFSET_V45) {
      Warning("cross page 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof (VMCrossPageV45), MODULECALL_CROSSPAGE_SIZE_V45,
              offsetof(VMCrossPageV45, crosspageLinearAddr),
              MODULECALL_CROSSPAGE_LAOFFSET_V45);
      goto error;
   }

   if (   sizeof (VMCrossPageV4) != MODULECALL_CROSSPAGE_SIZE_V4
       || offsetof(VMCrossPageV4, crosspageLinearAddr)
             != MODULECALL_CROSSPAGE_LAOFFSET_V4) {
      Warning("cross page 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof (VMCrossPageV4), MODULECALL_CROSSPAGE_SIZE_V4,
              offsetof(VMCrossPageV4, crosspageLinearAddr),
              MODULECALL_CROSSPAGE_LAOFFSET_V4);
      goto error;
   }

   if (sizeof(VMCrossPageGSX25) != MODULECALL_CROSSPAGE_SIZE_GSX25 ||
       offsetof(VMCrossPageGSX25,crosspageLinearAddr)!=MODULECALL_CROSSPAGE_LAOFFSET_GSX25) {
      Warning("cross page [gsx25] 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof(VMCrossPageGSX25), MODULECALL_CROSSPAGE_SIZE_GSX25,
              offsetof(VMCrossPageGSX25,crosspageLinearAddr), MODULECALL_CROSSPAGE_LAOFFSET_GSX25);
      goto error;
   }

   if (sizeof(VMCrossPageGSX2) != MODULECALL_CROSSPAGE_SIZE_GSX2 ||
       offsetof(VMCrossPageGSX2,crosspageLinearAddr)!=MODULECALL_CROSSPAGE_LAOFFSET_GSX2) {
      Warning("cross page [gsx2] 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof(VMCrossPageGSX2), MODULECALL_CROSSPAGE_SIZE_GSX2,
              offsetof(VMCrossPageGSX2,crosspageLinearAddr), MODULECALL_CROSSPAGE_LAOFFSET_GSX2);
      goto error;
   }

   if (sizeof(VMCrossPageV321) != MODULECALL_CROSSPAGE_SIZE_V321 ||
       offsetof(VMCrossPageV321,crosspageLinearAddr)!=MODULECALL_CROSSPAGE_LAOFFSET_V321) {
      Warning("cross page [v321] 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof(VMCrossPageV321), MODULECALL_CROSSPAGE_SIZE_V321,
              offsetof(VMCrossPageV321,crosspageLinearAddr), MODULECALL_CROSSPAGE_LAOFFSET_V321);
      goto error;
   }

   if (sizeof(VMCrossPageV32) != MODULECALL_CROSSPAGE_SIZE_V32 ||
       offsetof(VMCrossPageV32,crosspageLinearAddr)!=MODULECALL_CROSSPAGE_LAOFFSET_V32) {
      Warning("cross page [v32] 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof(VMCrossPageV32), MODULECALL_CROSSPAGE_SIZE_V32,
              offsetof(VMCrossPageV32,crosspageLinearAddr), MODULECALL_CROSSPAGE_LAOFFSET_V32);
      goto error;
   }

   if (sizeof(VMCrossPageV3) != MODULECALL_CROSSPAGE_SIZE_V3 ||
       offsetof(VMCrossPageV3,crosspageLinearAddr)!=MODULECALL_CROSSPAGE_LAOFFSET_V3) {
      Warning("cross page [v3] 0x%x expected 0x%x. la 0x%x expected 0x%x\n",
              sizeof(VMCrossPageV3), MODULECALL_CROSSPAGE_SIZE_V3,
              offsetof(VMCrossPageV3,crosspageLinearAddr), MODULECALL_CROSSPAGE_LAOFFSET_V3);
      goto error;
   }

   return TRUE;

error:
   return FALSE;
}
